require 'rspec'
require 'tempfile'

def write_to_tmp_file(file_content)
  tmpfile = Tempfile.new('rspec_tmp')
  tmpfile.write(file_content)
  tmpfile.close
  tmpfile
end

describe 'Configuration Loading' do

  before do
    $stdout = StringIO.new
    $stderr = StringIO.new
    @config_tree_test_file = File.dirname(__FILE__)+ '/../resources/host-tree-generation-test.yml'
  end
  after(:all) do
    $stdout = STDOUT
    $stderr = STDERR
  end

  it 'should fail with error if DockerMachine config file not found' do
    expect { Configurator::Config.new('fakefile') }.to raise_error(SystemExit, /Docker Machine configuration file not found: fakefile/)
  end
  it 'should fail with error if DockerMachine config file cannot be read' do
    tmpfile = Tempfile.new('fakefile')
    tmpfile.chmod(000)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, /Can't read Docker Machine configuration file: #{tmpfile.path}/)
  end
  it 'should fail with error if DockerCompose file not found' do
    #TODO
  end
  it 'should fail with error if DockerCompose file cannot be read' do
    #TODO
  end
  it 'should fail with error if driver is unknown' do
    file_content = "options:\n  driver: aws"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Invalid Docker Machine config: unknown driver 'aws'./)
  end
  it 'should fail with error if driver is amazonec2 and aws-credentials-profile is nil' do
    file_content = "options:\n  driver: amazonec2"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Invalid Docker Machine config: driver set as 'amazonec2', but 'aws-credentials-profile' parameter not set./)
  end
  it 'should fail with error if hosts section is not defined' do
    file_content = "options:\n"\
                         "  driver: amazonec2\n" \
                         "  aws-credentials-profile: default\n"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Invalid Docker Machine config: hosts are not defined./)
  end
  it 'should fail with error if host-type is not defined in host node' do
    file_content = "options:\n"\
                         "  driver: amazonec2\n" \
                         "  aws-credentials-profile: default\n"\
                         "hosts:\n"\
                         "  fail-host:\n"\
                         "    aparam: something"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: host-type is not defined for host 'fail-host'./)
  end
  it 'should fail with error if host type is unknown' do
    file_content = "options:\n"\
                         "  driver: amazonec2\n" \
                         "  aws-credentials-profile: default\n"\
                         "hosts:\n"\
                         "  fail-host:\n"\
                         "    host-type: something"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: unknown host-type 'something' for host 'fail-host'./)
  end
  it 'should fail with error if hosts-amount is not defined in host node' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: hosts-amount not defined for host 'fail-host'./)
  end
  it 'should fail with error if hosts-amount is not integer' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: fail"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: hosts-amount must be integer. Host: 'fail-host'./)
  end
  it 'should fail with error if AWS region not defined' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-region not defined for host 'fail-host'./)
  end
  it 'should fail with error if AWS region is not a string' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: \n"\
                       "      - test"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-region must be a string. Host: 'fail-host'./)
  end
  it 'should fail with error if AWS availability zone not defined' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-zone not defined for host 'fail-host'./)
  end
  it 'should fail with error if AWS availability zone is not a string' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: \n"\
                       "      - test"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-zone must be a string. Host: 'fail-host'./)
  end
  it 'should fail with error if AWS VPC ID not defined' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-vpc-id not defined for host 'fail-host'./)
  end
  it 'should fail with error if AWS VPC ID is not a string' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone\n"\
                       "    amazonec2-vpc-id:\n"\
                       "      - test"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-vpc-id must be a string. Host: 'fail-host'./)
  end
  it 'should fail with error if AWS Subnet ID not defined' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone\n"\
                       "    amazonec2-vpc-id: vpc"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-subnet-id not defined for host 'fail-host'./)
  end
  it 'should fail with error if AWS Subnet ID is not a string' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone\n"\
                       "    amazonec2-vpc-id: vpc\n"\
                       "    amazonec2-subnet-id:\n"\
                       "      - test"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-subnet-id must be a string. Host: 'fail-host'./)
  end
  it 'should fail with error if AWS Security Group not defined' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone\n"\
                       "    amazonec2-vpc-id: vpc\n"\
                       "    amazonec2-subnet-id: subnet"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-security-group not defined for host 'fail-host'./)
  end
  it 'should fail with error if AWS Security Group is not a string' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone\n"\
                       "    amazonec2-vpc-id: vpc\n"\
                       "    amazonec2-subnet-id: subnet\n"\
                       "    amazonec2-security-group:\n"\
                       "      - test"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-security-group must be a string. Host: 'fail-host'./)
  end
  it 'should fail with error if AWS Instance Type not defined' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone\n"\
                       "    amazonec2-vpc-id: vpc\n"\
                       "    amazonec2-subnet-id: subnet\n"\
                       "    amazonec2-security-group: sg"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-instance-type not defined for host 'fail-host'./)
  end
  it 'should fail with error if AWS Instance Type is not a string' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone\n"\
                       "    amazonec2-vpc-id: vpc\n"\
                       "    amazonec2-subnet-id: subnet\n"\
                       "    amazonec2-security-group: sg\n"\
                       "    amazonec2-instance-type:\n"\
                       "      - test"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-instance-type must be a string. Host: 'fail-host'./)
  end
  it 'should fail with error if AWS AMI is not a string' do
    file_content = "options:\n"\
                       "  driver: amazonec2\n" \
                       "  aws-credentials-profile: default\n"\
                       "hosts:\n"\
                       "  fail-host:\n"\
                       "    host-type: generic\n"\
                       "    hosts-amount: 1\n"\
                       "    amazonec2-region: region\n"\
                       "    amazonec2-zone: zone\n"\
                       "    amazonec2-vpc-id: vpc\n"\
                       "    amazonec2-subnet-id: subnet\n"\
                       "    amazonec2-security-group: sg\n"\
                       "    amazonec2-instance-type: t2.micro\n"\
                       "    amazonec2-ami:\n"\
                       "      - test"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
            /Invalid Docker Machine config: amazonec2-ami must be a string. Host: 'fail-host'./)
  end
  it 'should fail with error if driver is not defined in options section' do
    file_content = "driver: aws"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Wrong Docker Machine config format: options => driver parameter not found./)
  end
  it 'should fail with error if engine-opts is not an array' do
    file_content = "options:\n"\
                     "  driver: amazonec2\n" \
                     "  aws-credentials-profile: default\n"\
                     "hosts:\n"\
                     "  fail-host:\n"\
                     "    host-type: generic\n"\
                     "    hosts-amount: 2\n"\
                     "    engine-opts:\n" \
                     "      op1: sdf"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Invalid Docker Machine config: engine-opts must be an array in yml notation. Host: 'fail-host'./)
  end
  it 'should fail with error if commands-to-execute is not an array' do
    file_content = "options:\n"\
                     "  driver: amazonec2\n" \
                     "  aws-credentials-profile: default\n"\
                     "hosts:\n"\
                     "  fail-host:\n"\
                     "    host-type: generic\n"\
                     "    hosts-amount: 2\n"\
                     "    commands-to-execute:\n" \
                     "      op1: sdf"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Invalid Docker Machine config: commands-to-execute must be an array in yml notation. Host: 'fail-host'./)
  end
  it 'should fail with error if engine-install-url is not a string' do
    file_content = "options:\n"\
                     "  driver: amazonec2\n" \
                     "  aws-credentials-profile: default\n"\
                     "hosts:\n"\
                     "  fail-host:\n"\
                     "    host-type: generic\n"\
                     "    hosts-amount: 2\n"\
                     "    engine-install-url:\n" \
                     "      op1: sdf"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Invalid Docker Machine config: engine-install-url must be a string. Host: 'fail-host'./)
  end
  it 'should fail with error if host-type is "swarm-node" and swarm-discovery is not defined' do
    file_content = "options:\n"\
                     "  driver: amazonec2\n" \
                     "  aws-credentials-profile: default\n"\
                     "hosts:\n"\
                     "  fail-host:\n"\
                     "    host-type: swarm-node\n"\
                     "    hosts-amount: 2"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Invalid Docker Machine config: host-type is swarm-node, but swarm-discovery is not set. Host: 'fail-host'./)
  end
  it 'should fail with error if host-type is "swarm-node" and swarm-discovery is not a string' do
    file_content = "options:\n"\
                     "  driver: amazonec2\n" \
                     "  aws-credentials-profile: default\n"\
                     "hosts:\n"\
                     "  fail-host:\n"\
                     "    host-type: swarm-node\n"\
                     "    hosts-amount: 2\n"\
                     "    swarm-discovery:\n" \
                     "      op1: sdf"
    tmpfile = write_to_tmp_file(file_content)
    expect { Configurator::Config.new(tmpfile.path) }.to raise_error(SystemExit, \
          /Invalid Docker Machine config: swarm-discovery must be a string. Host: 'fail-host'./)
  end
  it 'should return proper creation string for amazonec2 generic host type' do
    test_creation_string = "--driver amazonec2 "\
        "--engine-install-url 'http://test.com' --engine-opt='opt1' --engine-opt='opt2' "\
        "--amazonec2-region region --amazonec2-zone zone --amazonec2-vpc-id vpc "\
        "--amazonec2-subnet-id subnet --amazonec2-security-group sg --amazonec2-instance-type it"
    config = Configurator::Config.new(@config_tree_test_file)
    expect(config.config_tree['generic-host'].get_creation_string).to eq(test_creation_string)
  end
  it 'should return proper creation string for swarm-node host type' do
    test_creation_string = "--driver amazonec2 "\
        "--engine-install-url 'http://test.com' --engine-opt='opt1' --engine-opt='opt2' "\
        "--swarm --swarm-discovery='something_somewhere' "\
        "--amazonec2-region region --amazonec2-zone zone --amazonec2-vpc-id vpc "\
        "--amazonec2-subnet-id subnet --amazonec2-security-group sg --amazonec2-instance-type it"
    config = Configurator::Config.new(@config_tree_test_file)
    expect(config.config_tree['swarm-node'].get_creation_string).to eq(test_creation_string)
  end
  it 'should return proper creation string for swarm-master host type' do
    test_creation_string = "--driver amazonec2 "\
        "--engine-install-url 'http://test.com' --engine-opt='opt1' --engine-opt='opt2' "\
        "--swarm-master --swarm-discovery='something_somewhere' "\
        "--amazonec2-region region --amazonec2-zone zone --amazonec2-vpc-id vpc "\
        "--amazonec2-subnet-id subnet --amazonec2-security-group sg --amazonec2-instance-type it"
    config = Configurator::Config.new(@config_tree_test_file)
    expect(config.config_tree['swarm-master'].get_creation_string).to eq(test_creation_string)
  end
end